import sys
import codecs
from xml.dom.minidom import parse
from xml.dom.minidom import Element

layers = {}

def get_attr(item, name, value):
    if item.hasAttribute(name):
        return item.getAttribute(name)
    else:
        if value == None:
            raise RuntimeError(name+' not specific')
        return value


def get_name(item):
    name = get_attr(item, 'name', '')
    if len(name) == 0:
        raise RuntimeError('name not specific')
    return name


def write_create_bmp_func(file):
    impl = """static P_Layer create_bmp(uint8_t x, uint8_t y, uint8_t w, uint8_t h, uint32_t bmpkey) {
    GBitmap pixel;
    res_get_user_bitmap(bmpkey, &pixel);
    LayerBitmap bmp;
    bmp.bitmap = pixel;
    bmp.frame.origin.x = x;
    bmp.frame.origin.y = y;
    bmp.frame.size.w = w;
    bmp.frame.size.h = h;
    bmp.alignment = GAlignCenter;
    P_Layer layer = app_layer_create_bitmap(&bmp);
    return layer;
}

"""
    file.write(impl)


def write_create_text_func(file):
    impl = """static P_Layer create_text(uint8_t x, uint8_t y, uint8_t w, uint8_t h, const char* str, uint8_t align, uint8_t font, uint8_t bound, uint8_t bg) {
    LayerText text;
    text.text = str;
    text.frame.origin.x = x;
    text.frame.origin.y = y;
    text.frame.size.w = w;
    text.frame.size.h = h;
    text.alignment = align;
    text.font_type = font;
    text.bound_width = bound;
    P_Layer layer = app_layer_create_text(&text);
    if (layer) app_layer_set_bg_color(layer, bg);
    return layer;
}

"""
    file.write(impl)


def convert_rect(item):
    x = get_attr(item, 'x', '0')
    y = get_attr(item, 'y', '0')
    w = get_attr(item, 'w', '0')
    h = get_attr(item, 'h', '0')

    impl = []
    impl.append('   GRect rect;')
    impl.append('   rect.origin.x = ' + x +';')
    impl.append('   rect.origin.y = ' + y +';')
    impl.append('   rect.size.w = ' + w +';')
    impl.append('   rect.size.h = ' + h +';')

    return impl


def get_frame(item):
    return [get_attr(item, 'x', '0'), get_attr(item, 'y', '0'), get_attr(item, 'w', '0'), get_attr(item, 'h', '0')]


def define_create_layer_func(args, name, sub):
    impl = []
    impl.append('static P_Layer '+name+'() {')
    impl.append('   return '+sub+'('+','.join(args)+');')
    impl.append('}\n\n')
    return '\n'.join(impl)


def convert_text(item):
    name = get_name(item)
    access = '' 
    if get_attr(item, 'access', '1') == '1':
        access = 'g_'+name+'_textid'
    layers[name] = [access, 'create_'+name+'_text', False]

    args = []
    args.extend(get_frame(item))
    args.append('"'+get_attr(item, 'text', '')+'"')
    args.append('GAlign'+get_attr(item, 'align', 'Center'))
    args.append(get_attr(item, 'font', 'U_ASCII_ARIAL_12'))
    args.append(get_attr(item, 'bound', '0'))
    args.append('GColor'+get_attr(item, 'bg', 'White'))

    return define_create_layer_func(args, layers[name][1], 'create_text')


def convert_statusbar(item):
    show = get_attr(item, 'show', '')
    impl = []
    impl.append('static void create_' + get_name(item) + '_status_bar(P_Window window) {')
    impl.append('   app_plug_status_bar_create(window, NULL, NULL, NULL);')
    if len(show) > 0:
        modes = show.split('|')
        for mode in modes:
            impl.append('   app_plug_status_bar_add_' + mode + '(window);')
    impl.append('}\n\n')

    return '\n'.join(impl)


def convert_bmp(item):
    name = get_name(item)
    access = '' 
    if get_attr(item, 'access', '1') == '1':
        access = 'g_'+name+'_bmpid'
    layers[name] = [access, 'create_'+name+'_bmp', False]

    args = []
    args.extend(get_frame(item))
    args.append(get_attr(item, 'src', None))
    
    return define_create_layer_func(args, layers[name][1], 'create_bmp')


def write_header(file):
    file.write('#include "maibu_res.h"\n')
    file.write('#include <maibu_sdk.h>\n\n')


def write_create_window(file, name):
    for key in layers:
        layer_id = layers[key][0]
        if len(layer_id) > 0:
            file.write('static uint8_t '+layer_id+' = 0;\n')
    
    file.write('\n')

    file.write('static P_Window create_'+name+'_window() {\n')
    file.write('    P_Window window = app_window_create();\n')
    for key in layers:  
        layer_id = layers[key][0]
        layer_func = layers[key][1]
        layer_param = layers[key][2]

        file.write('    ')
        if len(layer_id) > 0:
            file.write(layer_id + ' = ')
        file.write('app_window_add_layer(window, '+layer_func+'(')
        if layer_param:
            file.write('window')
        file.write('));\n')
    file.write('    return window;\n')
    file.write('}\n')


def convert_window(win, file):
    for item in win.childNodes:
        if not isinstance(item, Element):
            continue

        if item.tagName == 'text':
            file.write(convert_text(item))
        elif item.tagName == 'statusbar':
            file.write(convert_statusbar(item))
        elif item.tagName == 'bmp':
            file.write(convert_bmp(item))
        else:
            file.write('//unsupported ui item: '+item.tagName)

    write_create_window(file, get_name(win))


def convert(xml, c):
    global layers
    file = codecs.open(c, 'w', 'utf-8')
    write_header(file)
    write_create_text_func(file)
    write_create_bmp_func(file)

    doc = parse(xml)
    wins = doc.getElementsByTagName('window')
    for win in wins:
        layers={}
        convert_window(win, file)
        
    file.close()


if __name__ == '__main__':
    if len(sys.argv) < 3:
        print('usage: uiconvert.py ui.xml ui.c')
        exit(-1)

    xml = sys.argv[1]
    c = sys.argv[2]
    convert(xml, c)